<?php

namespace App\Controller;

use App\Constants;
use Artprima\QueryFilterBundle\QueryFilter\Config\BaseConfig;
use Artprima\QueryFilterBundle\QueryFilter\QueryFilter;
use Artprima\QueryFilterBundle\Response\Response;
use Artprima\QueryFilterBundle\Request\Request;


class BaseController
{

    protected function getList($repository, $request, $sortColumn, $columns): \Artprima\QueryFilterBundle\Response\ResponseInterface
    {
        // TODO: Implement __invoke() method.
        $orderBy = ($request->get('orderBy') && $request->get('orderBy') != null)?$request->get('orderBy'):$sortColumn;
        $sortOrder = ($request->get('sortOrder') && $request->get('sortOrder') != null)?$request->get('sortOrder'):'desc';
        $limit = ($request->get('itemsperpage') && $request->get('itemsperpage') != null)?$request->get('itemsperpage'):50;

        // set up the config
        $config = new BaseConfig();
        $config->setSearchAllowedCols($columns);
        $config->setSearchAllowedColsLengths([$orderBy => ['min' => 1, 'max' => 255]]);
        $config->setAllowedLimits([10, 25, 50, 100]);
        $config->setDefaultLimit($limit);
        $config->setSortCols($columns, [$orderBy => $sortOrder]);
        $config->setRequest(new Request($request));

        // Throws an UnexpectedValueException when invalid filter column, sort column or sort type is specified
        $config->setStrictColumns(true);

        // here we provide a repository callback that will be used internally in the QueryFilter
        // The signature of the method must be as follows: function functionName(QueryFilterArgs $args): QueryResult;
        $config->setRepositoryCallback([$repository, 'findByOrderBy']);


        // Response must implement Artprima\QueryFilterBundle\Response\ResponseInterface
        $queryFilter = new QueryFilter(Response::class);
        /** @var Response $data the type of the variable is defined by the class in the first argument of QueryFilter's constructor */
        $response = $queryFilter->getData($config);

        return $response;
    }

}