<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\activite\CreateActivite;
use App\Controller\activite\ListeActivite;
use App\Repository\ActiviteRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: ActiviteRepository::class)]
#[ApiResource(
    collectionOperations: [
          'liste_activite' => [
            'method' => 'GET',
            'path' => '/activites',
            'controller' => ListeActivite::class
           ],
           'create_activite' => [
            'method' => 'POST',
            'path' => '/activites',
            'controller' => CreateActivite::class
           ]
           
      
],
    itemOperations: ['get','put','delete'],
    attributes:['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    normalizationContext: ['groups' => ['act:read'],'enable_max_depth' => true, // If using Max Depth
                                                      'skip_null_values' => false ],

)]
class Activite
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?string $nom = null;



    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?string $description = null;

    #[ORM\Column]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\OneToMany(mappedBy: 'activite', targetEntity: Usager::class)]
    #[Groups(['act:read'])]
    private Collection $usagers;

    #[ORM\OneToMany(mappedBy: 'activite', targetEntity: BaseRedevable::class)]
    private Collection $baseRedevables;

    #[ORM\OneToMany(mappedBy: 'activite', targetEntity: Enrollement::class)]
    private Collection $enrollements;

    #[ORM\Column(length: 255)]
    #[Groups(['act:read','usager:read','bsr:read'])]

    private ?string $code = null;

    public function __construct()
    {
        $this->usagers = new ArrayCollection();
        $this->baseRedevables = new ArrayCollection();
        $this->enrollements = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): static
    {
        $this->description = $description;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, Usager>
     */
    public function getUsagers(): Collection
    {
        return $this->usagers;
    }

    public function addUsager(Usager $usager): static
    {
        if (!$this->usagers->contains($usager)) {
            $this->usagers->add($usager);
            $usager->setActivite($this);
        }

        return $this;
    }

    public function removeUsager(Usager $usager): static
    {
        if ($this->usagers->removeElement($usager)) {
            // set the owning side to null (unless already changed)
            if ($usager->getActivite() === $this) {
                $usager->setActivite(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, BaseRedevable>
     */
    public function getBaseRedevables(): Collection
    {
        return $this->baseRedevables;
    }

    public function addBaseRedevable(BaseRedevable $baseRedevable): static
    {
        if (!$this->baseRedevables->contains($baseRedevable)) {
            $this->baseRedevables->add($baseRedevable);
            $baseRedevable->setActivite($this);
        }

        return $this;
    }

    public function removeBaseRedevable(BaseRedevable $baseRedevable): static
    {
        if ($this->baseRedevables->removeElement($baseRedevable)) {
            // set the owning side to null (unless already changed)
            if ($baseRedevable->getActivite() === $this) {
                $baseRedevable->setActivite(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Enrollement>
     */
    public function getEnrollements(): Collection
    {
        return $this->enrollements;
    }

    public function addEnrollement(Enrollement $enrollement): static
    {
        if (!$this->enrollements->contains($enrollement)) {
            $this->enrollements->add($enrollement);
            $enrollement->setActivite($this);
        }

        return $this;
    }

    public function removeEnrollement(Enrollement $enrollement): static
    {
        if ($this->enrollements->removeElement($enrollement)) {
            // set the owning side to null (unless already changed)
            if ($enrollement->getActivite() === $this) {
                $enrollement->setActivite(null);
            }
        }

        return $this;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): static
    {
        $this->code = $code;

        return $this;
    }
}
