<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\application\CreateApplication;
use App\Controller\application\ListeApplication;
use App\Repository\ApplicationRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: ApplicationRepository::class)]
#[ApiResource(
    collectionOperations: [
          'liste_application' => [
            'method' => 'GET',
            'path' => '/applications',
            'controller' => ListeApplication::class
           ],
           'create_application' => [
            'method' => 'POST',
            'path' => '/applications',
            'controller' => CreateApplication::class
           ]
           
      
],
    itemOperations: ['get','put','delete'],
    attributes: ['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    denormalizationContext: ['groups' => ['app:write', 'bsr:write']],
    normalizationContext: ['groups' => ['app:read', 'bsr:read', 'ram:details:read'],'enable_max_depth' => true, // If using Max Depth
                                                      'skip_null_values' => false ]

)]
class Application
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['app:read','pr:read','ville:read', 'app:write', 'bsr:write'])]
    private ?int $id = null;

    #[ORM\ManyToOne(inversedBy: 'applications')]
    #[Groups(['app:read','app:write','pr:read', 'app:write', 'bsr:write'])]
    private ?Ville $ville = null;

    #[ORM\Column(type: Types::DECIMAL, precision: 10, scale: '0', nullable: true)]
    #[Groups(['app:read','app:write','pr:read','ville:read', 'app:write', 'bsr:write'])]
    private ?string $tarif = null;

   

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['app:read','app:write','pr:read','ville:read', 'app:write', 'bsr:write'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['app:read','app:write','pr:read','ville:read', 'app:write', 'bsr:write'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\ManyToMany(targetEntity: BaseRedevable::class, mappedBy: 'applications', cascade: ['persist', 'remove'])]
    private Collection $baseRedevables;

    #[ORM\ManyToOne(inversedBy: 'applications')]
    #[Groups(['app:read','ram:details:read', 'app:write', 'bsr:write'])]

    private ?Ramification $ramification = null;

    #[ORM\Column(length: 255)]
    #[Groups(['app:read', 'app:write', 'bsr:write'])]
    
    private ?string $annee = null;

    #[ORM\ManyToMany(targetEntity: Precision::class, inversedBy: 'applications', cascade: ['persist', 'remove'])]
    #[Groups(['app:read','bsr:write','app:write','pr:read', 'bsr:read'])]

    private Collection $precisions;

    public function __construct()
    {
        $this->baseRedevables = new ArrayCollection();
        $this->precisions = new ArrayCollection();
        $this->createdAt = new \datetime;
        $this->updatedAt = new \datetime;
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getVille(): ?Ville
    {
        return $this->ville;
    }

    public function setVille(?Ville $ville): static
    {
        $this->ville = $ville;

        return $this;
    }

    public function getTarif(): ?string
    {
        return $this->tarif;
    }

    public function setTarif(?string $tarif): static
    {
        $this->tarif = $tarif;

        return $this;
    }

    

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, BaseRedevable>
     */
    public function getBaseRedevables(): Collection
    {
        return $this->baseRedevables;
    }

    public function addBaseRedevable(BaseRedevable $baseRedevable): static
    {
        if (!$this->baseRedevables->contains($baseRedevable)) {
            $this->baseRedevables->add($baseRedevable);
            $baseRedevable->addApplication($this);
        }

        return $this;
    }

    public function removeBaseRedevable(BaseRedevable $baseRedevable): static
    {
        if ($this->baseRedevables->removeElement($baseRedevable)) {
            $baseRedevable->removeApplication($this);
        }

        return $this;
    }

    public function getRamification(): ?Ramification
    {
        return $this->ramification;
    }

    public function setRamification(?Ramification $ramification): static
    {
        $this->ramification = $ramification;

        return $this;
    }

    public function getAnnee(): ?string
    {
        return $this->annee;
    }

    public function setAnnee(string $annee): static
    {
        $this->annee = $annee;

        return $this;
    }

    /**
     * @return Collection<int, Precision>
     */
    public function getPrecisions(): Collection
    {
        return $this->precisions;
    }

    public function addPrecision(Precision $precision): static
    {
        if (!$this->precisions->contains($precision)) {
            $this->precisions->add($precision);
        }

        return $this;
    }

    public function removePrecision(Precision $precision): static
    {
        $this->precisions->removeElement($precision);

        return $this;
    }
}
