<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\baseRedevable\CreateBaseRedevable;
use App\Controller\baseRedevable\ListeBaseRedevable;
use App\Repository\BaseRedevableRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: BaseRedevableRepository::class)]
#[ApiResource(
    collectionOperations: [
          'liste_base_redevable' => [
            'method' => 'GET',
            'path' => '/base_redevables',
            'controller' => ListeBaseRedevable::class
           ],
           'create_base_redevable' => [
            'method' => 'POST',
            'path' => '/base_redevables',
            'controller' => CreateBaseRedevable::class
           ]
           
      
],
    itemOperations: ['get','put','delete'],
    attributes: ['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    denormalizationContext: ['groups' => ['bsr:write']],
    normalizationContext: ['groups' => ['bsr:read', 'frq:details:read','act:read','app:read'],'enable_max_depth' => true, // If using Max Depth
                                                      'skip_null_values' => false ],

)]
class BaseRedevable
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['bsr:read','bsr:write'])]
    private ?int $id = null;

    #[ORM\ManyToOne(inversedBy: 'baseRedevables')]
    #[Groups(['bsr:read','act:read','bsr:write'])]
    private ?Activite $activite = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['bsr:read','bsr:write'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['bsr:read','bsr:write'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\Column]
    #[Groups(['bsr:read','bsr:write'])]
    private ?bool $enabled = null;

    #[ORM\ManyToOne(inversedBy: 'baseRedevables')]
    #[Groups(['bsr:read', 'frq:details:read','bsr:write'])]
    private ?Frequence $frequence = null;

    #[ORM\Column(length: 255)]
    #[Groups(['bsr:read','bsr:write'])]

    private ?string $nom = null;

    #[ORM\OneToMany(mappedBy: 'redevance', targetEntity: Echeance::class)]
    private Collection $echeances;

    #[ORM\ManyToMany(targetEntity: Application::class, inversedBy: 'baseRedevables', cascade: ['persist', 'remove'])]
    #[Groups([ 'bsr:read', 'bsr:write','app:read'])]

    private Collection $applications;

    public function __construct()
    {
        $this->echeances = new ArrayCollection();
        $this->applications = new ArrayCollection();
        $this->createdAt = new \datetime;
        $this->updatedAt = new \datetime;
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getActivite(): ?Activite
    {
        return $this->activite;
    }

    public function setActivite(?Activite $activite): static
    {
        $this->activite = $activite;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getFrequence(): ?Frequence
    {
        return $this->frequence;
    }

    public function setFrequence(?Frequence $frequence): static
    {
        $this->frequence = $frequence;

        return $this;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    /**
     * @return Collection<int, Echeance>
     */
    public function getEcheances(): Collection
    {
        return $this->echeances;
    }

    public function addEcheance(Echeance $echeance): static
    {
        if (!$this->echeances->contains($echeance)) {
            $this->echeances->add($echeance);
            $echeance->setRedevance($this);
        }

        return $this;
    }

    public function removeEcheance(Echeance $echeance): static
    {
        if ($this->echeances->removeElement($echeance)) {
            // set the owning side to null (unless already changed)
            if ($echeance->getRedevance() === $this) {
                $echeance->setRedevance(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Application>
     */
    public function getApplications(): Collection
    {
        return $this->applications;
    }

    public function addApplication(Application $application): static
    {
        if (!$this->applications->contains($application)) {
            $this->applications->add($application);
        }

        return $this;
    }

    public function removeApplication(Application $application): static
    {
        $this->applications->removeElement($application);

        return $this;
    }
}
