<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\echeances\CreateEcheance;
use App\Controller\echeances\ListEcheance;
use App\Controller\usager\RedevableEcheances;
use App\Repository\EcheanceRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: EcheanceRepository::class)]
#[ApiResource(
    collectionOperations: [
        'redevable_echeance' => [
            'method' => 'GET',
            'path' => '/usagers/{id}/echeances',
            'controller' => RedevableEcheances::class
        ],

    ],
    itemOperations: [
        'get',
        'delete',
        'put'

    ],
    attributes: ["route_prefix"=>"/v1", "pagination_client_items_per_page"=>true],
    normalizationContext: ["skip_null_values" => false,'groups' => ['ech:read', 'ech:details:read', 'trans:details:read', 'usager:read', 'enr:details:read', 'tax:details:read', 'bsr:read']]
)]
class Echeance
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?int $id = null;

    #[ORM\ManyToOne(inversedBy: 'echeances')]
    #[Groups([ 'ech:read', 'ech:details:read', 'bsr:read'])]

    private ?BaseRedevable $redevance = null;

    #[ORM\Column(type: Types::DECIMAL, precision: 10, scale: '0')]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?string $montant = null;

    #[ORM\ManyToOne(inversedBy: 'echeances')]
    #[Groups([ 'ech:read', 'ech:details:read', 'usager:read'])]

    private ?Usager $usager = null;

    #[ORM\ManyToOne(inversedBy: 'echeances')]
    private ?Enrollement $enrollement = null;

    #[ORM\Column]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?bool $isPaid = null;

    #[Groups([ 'ech:read', 'ech:details:read'])]
    private $situation;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\Column(length: 255)]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?string $annee = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?string $mois = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private ?string $jour = null;

    #[ORM\OneToMany(mappedBy: 'echeance', targetEntity: Transaction::class)]
    #[Groups([ 'ech:read', 'ech:details:read'])]

    private Collection $transactions;

    #[ORM\OneToMany(mappedBy: 'echeance', targetEntity: Paiement::class)]
    private Collection $paiements;


    public function __construct()
    {
        $this->transactions = new ArrayCollection();
        $this->paiements = new ArrayCollection();

    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getRedevance(): ?BaseRedevable
    {
        return $this->redevance;
    }

    public function setRedevance(?BaseRedevable $redevance): static
    {
        $this->redevance = $redevance;

        return $this;
    }

    public function getMontant(): ?string
    {
        return $this->montant;
    }

    public function setMontant(string $montant): static
    {
        $this->montant = $montant;

        return $this;
    }

    public function getUsager(): ?Usager
    {
        return $this->usager;
    }

    public function setUsager(?Usager $usager): static
    {
        $this->usager = $usager;

        return $this;
    }

    public function getEnrollement(): ?Enrollement
    {
        return $this->enrollement;
    }

    public function setEnrollement(?Enrollement $enrollement): static
    {
        $this->enrollement = $enrollement;

        return $this;
    }

    public function isIsPaid(): ?bool
    {
        return $this->isPaid;
    }

    public function setIsPaid(bool $isPaid): static
    {
        $this->isPaid = $isPaid;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getAnnee(): ?string
    {
        return $this->annee;
    }

    public function setAnnee(string $annee): static
    {
        $this->annee = $annee;

        return $this;
    }

    public function getMois(): ?string
    {
        return $this->mois;
    }

    public function setMois(?string $mois): static
    {
        $this->mois = $mois;

        return $this;
    }

    public function getJour(): ?string
    {
        return $this->jour;
    }

    public function setJour(?string $jour): static
    {
        $this->jour = $jour;

        return $this;
    }

    /**
     * @return Collection<int, Transaction>
     */
    public function getTransactions(): Collection
    {
        return $this->transactions;
    }

    public function addTransaction(Transaction $transaction): static
    {
        if (!$this->transactions->contains($transaction)) {
            $this->transactions->add($transaction);
            $transaction->setEcheance($this);
        }

        return $this;
    }

    public function removeTransaction(Transaction $transaction): static
    {
        if ($this->transactions->removeElement($transaction)) {
            // set the owning side to null (unless already changed)
            if ($transaction->getEcheance() === $this) {
                $transaction->setEcheance(null);
            }
        }

        return $this;
    }

    public function getSituation(){
        $payer = 0;
        $restant = 0;
        foreach ($this->getPaiements() as $paiement){
            foreach ($paiement->getTransactions() as $transaction){
                if ($transaction->getStatut() == 'Confirmé')
                    $payer+= $paiement->getMontant();
            }

        }

        $restant = $this->getMontant() - $payer;

        return [
            'payer' => $payer,
            'restant' => $restant,

        ];

    }
    /**
     * @return Collection<int, Paiement>
     */
    public function getPaiements(): Collection
    {
        return $this->paiements;
    }

    public function addPaiement(Paiement $paiement): static
    {
        if (!$this->paiements->contains($paiement)) {
            $this->paiements->add($paiement);
            $paiement->setEcheance($this);
        }

        return $this;
    }

    public function removePaiement(Paiement $paiement): static
    {
        if ($this->paiements->removeElement($paiement)) {
            // set the owning side to null (unless already changed)
            if ($paiement->getEcheance() === $this) {
                $paiement->setEcheance(null);
            }
        }

        return $this;
    }
}
