<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\activite\CreateActivite;
use App\Controller\activite\ListeActivite;
use App\Controller\enrollements\CreateEnrollement;
use App\Repository\ActiviteRepository;
use App\Repository\EnrollementRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: EnrollementRepository::class)]
#[ApiResource(
    collectionOperations: [
        'liste_activite' => [
            'method' => 'GET',
            'path' => '/enrollements',
            'controller' => ListeActivite::class
        ],
        'create_activite' => [
            'method' => 'POST',
            'path' => '/enrollements',
            'controller' => CreateEnrollement::class
        ]


    ],
    itemOperations: ['get','put','delete'],
    attributes:['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    normalizationContext: ['groups' => ['enr:read', 'enr:details:read','usager:read','act:read','user:read'],'enable_max_depth' => true, // If using Max Depth
        'skip_null_values' => false ]

)]
class Enrollement
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups([ 'enr:read', 'enr:details:read'])]

    private ?int $id = null;

    #[ORM\ManyToOne(inversedBy: 'enrollements')]
    #[Groups([ 'enr:read', 'enr:details:read','usager:read'])]

    private ?Usager $usager = null;

    #[ORM\ManyToOne(inversedBy: 'enrollements')]
    #[Groups([ 'enr:read', 'enr:details:read', 'act:read'])]

    private ?Activite $activite = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'enr:read', 'enr:details:read'])]

    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'enr:read', 'enr:details:read'])]

    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\ManyToOne(inversedBy: 'enrollements')]
    #[Groups([ 'enr:read', 'enr:details:read','user:read'])]

    private ?User $createdBy = null;

    #[ORM\Column]
    #[Groups([ 'enr:read', 'enr:details:read'])]

    private ?bool $enabled = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $montant = null;

    #[ORM\OneToMany(mappedBy: 'enrollement', targetEntity: Echeance::class)]
    private Collection $echeances;

    public function __construct()
    {
        $this->echeances = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getUsager(): ?Usager
    {
        return $this->usager;
    }

    public function setUsager(?Usager $usager): static
    {
        $this->usager = $usager;

        return $this;
    }

    public function getActivite(): ?Activite
    {
        return $this->activite;
    }

    public function setActivite(?Activite $activite): static
    {
        $this->activite = $activite;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getCreatedBy(): ?User
    {
        return $this->createdBy;
    }

    public function setCreatedBy(?User $createdBy): static
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getMontant(): ?string
    {
        return $this->montant;
    }

    public function setMontant(?string $montant): static
    {
        $this->montant = $montant;

        return $this;
    }

    /**
     * @return Collection<int, Echeance>
     */
    public function getEcheances(): Collection
    {
        return $this->echeances;
    }

    public function addEcheance(Echeance $echeance): static
    {
        if (!$this->echeances->contains($echeance)) {
            $this->echeances->add($echeance);
            $echeance->setEnrollement($this);
        }

        return $this;
    }

    public function removeEcheance(Echeance $echeance): static
    {
        if ($this->echeances->removeElement($echeance)) {
            // set the owning side to null (unless already changed)
            if ($echeance->getEnrollement() === $this) {
                $echeance->setEnrollement(null);
            }
        }

        return $this;
    }
}
