<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\frequences\CreateFrequence;
use App\Controller\frequences\ListeFrequences;
use App\Controller\taxes\CreateTaxes;
use App\Controller\taxes\ListeTaxes;
use App\Repository\FrequenceRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: FrequenceRepository::class)]
#[ApiResource(
    collectionOperations: [
        'list_frequences' => [
            'method' => 'GET',
            'path' => '/frequences',
            'controller' => ListeFrequences::class,
        ],
        'create_frequences' => [
            'method' => 'POST',
            'path' => '/frequences',
            'controller' => CreateFrequence::class,
        ],

    ],
    itemOperations: [
        'get',
        'delete',
        'put'

    ],
    attributes: ["route_prefix"=>"/v1", "pagination_client_items_per_page"=>true],
    normalizationContext: ["skip_null_values" => false,'groups' => ['frq:read', 'frq:details:read']]
)]
class Frequence
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups([ 'frq:read', 'frq:details:read'])]

    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups([ 'frq:read', 'frq:details:read'])]

    private ?string $nom = null;

    #[ORM\Column]
    #[Groups([ 'frq:read', 'frq:details:read'])]

    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'frq:read', 'frq:details:read'])]

    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'frq:read', 'frq:details:read'])]

    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\OneToMany(mappedBy: 'frequence', targetEntity: BaseRedevable::class)]
    private Collection $baseRedevables;



    public function __construct()
    {
        $this->baseRedevables = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, BaseRedevable>
     */
    public function getBaseRedevables(): Collection
    {
        return $this->baseRedevables;
    }

    public function addBaseRedevable(BaseRedevable $baseRedevable): static
    {
        if (!$this->baseRedevables->contains($baseRedevable)) {
            $this->baseRedevables->add($baseRedevable);
            $baseRedevable->setFrequence($this);
        }

        return $this;
    }

    public function removeBaseRedevable(BaseRedevable $baseRedevable): static
    {
        if ($this->baseRedevables->removeElement($baseRedevable)) {
            // set the owning side to null (unless already changed)
            if ($baseRedevable->getFrequence() === $this) {
                $baseRedevable->setFrequence(null);
            }
        }

        return $this;
    }


}
