<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\moyen_paiements\CreateMoyenPaiement;
use App\Controller\moyen_paiements\ListeMoyenPaiement;
use App\Repository\MoyenPaiementRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: MoyenPaiementRepository::class)]
#[ApiResource(
    collectionOperations: [
        'liste_pays' => [
            'method' => 'GET',
            'path' => '/moyen_paiements',
            'controller' => ListeMoyenPaiement::class
        ],
        'create_pays' => [
            'method' => 'POST',
            'path' => '/moyen_paiements',
            'controller' => CreateMoyenPaiement::class
        ]


    ],
    itemOperations: ['get','put','delete'],
    attributes: ['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    normalizationContext: ['groups' => ['mp:read'],'enable_max_depth' => true, // If using Max Depth
        'skip_null_values' => false ]

)]
class MoyenPaiement
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['mp:read'])]

    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups(['mp:read'])]

    private ?string $nom = null;

    #[ORM\Column]
    #[Groups(['mp:read'])]

    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['mp:read'])]

    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['mp:read'])]

    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\OneToMany(mappedBy: 'moyenPaiement', targetEntity: Transaction::class)]
    private Collection $transactions;

    public function __construct()
    {
        $this->transactions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, Transaction>
     */
    public function getTransactions(): Collection
    {
        return $this->transactions;
    }

    public function addTransaction(Transaction $transaction): static
    {
        if (!$this->transactions->contains($transaction)) {
            $this->transactions->add($transaction);
            $transaction->setMoyenPaiement($this);
        }

        return $this;
    }

    public function removeTransaction(Transaction $transaction): static
    {
        if ($this->transactions->removeElement($transaction)) {
            // set the owning side to null (unless already changed)
            if ($transaction->getMoyenPaiement() === $this) {
                $transaction->setMoyenPaiement(null);
            }
        }

        return $this;
    }
}
