<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\contribuables\SendSMS;
use App\Controller\statistics\DashboardCollecteChart;
use App\Controller\statistics\DashboardPieCollecteChart;
use App\Controller\transactions\CheckTransaction;
use App\Controller\transactions\CollecteSituation;
use App\Controller\transactions\CreateCollecte;
use App\Controller\transactions\CreatePaiement;
use App\Controller\transactions\CreateTransaction;
use App\Controller\transactions\DashboardChart;
use App\Controller\transactions\DashboardPieChart;
use App\Controller\transactions\ListeCollecte;
use App\Controller\transactions\ListeTransactions;
use App\Repository\PaiementRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity(repositoryClass: PaiementRepository::class)]
#[ApiResource(
    collectionOperations: [

        'create_transactions' => [
            'method' => 'POST',
            'path' => '/paiements',
            'controller' => CreatePaiement::class,
        ],


    ],
    itemOperations: [
        'get',
        'delete',
        'put'

    ],
    attributes: ["route_prefix"=>"/v1", "pagination_client_items_per_page"=>true],
    normalizationContext: ["skip_null_values" => false]
)]
class Paiement
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    private ?int $id = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\ManyToOne(inversedBy: 'paiements')]
    private ?Echeance $echeance = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $terminal = null;

    #[ORM\Column]
    private ?bool $isPaid = false;

    #[ORM\ManyToOne(inversedBy: 'paiements')]
    private ?User $createdBy = null;

    #[ORM\ManyToMany(targetEntity: Transaction::class, inversedBy: 'paiements')]
    private Collection $transactions;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $montant = null;



    public function __construct()
    {
        $this->transactions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getEcheance(): ?Echeance
    {
        return $this->echeance;
    }

    public function setEcheance(?Echeance $echeance): static
    {
        $this->echeance = $echeance;

        return $this;
    }

    public function getTerminal(): ?string
    {
        return $this->terminal;
    }

    public function setTerminal(?string $terminal): static
    {
        $this->terminal = $terminal;

        return $this;
    }

    public function isIsPaid(): ?bool
    {
        return $this->isPaid;
    }

    public function setIsPaid(bool $isPaid): static
    {
        $this->isPaid = $isPaid;

        return $this;
    }

    public function getCreatedBy(): ?User
    {
        return $this->createdBy;
    }

    public function setCreatedBy(?User $createdBy): static
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    /**
     * @return Collection<int, Transaction>
     */
    public function getTransactions(): Collection
    {
        return $this->transactions;
    }

    public function addTransaction(Transaction $transaction): static
    {
        if (!$this->transactions->contains($transaction)) {
            $this->transactions->add($transaction);
        }

        return $this;
    }

    public function removeTransaction(Transaction $transaction): static
    {
        $this->transactions->removeElement($transaction);

        return $this;
    }

    public function getMontant(): ?string
    {
        return $this->montant;
    }

    public function setMontant(string $montant): static
    {
        $this->montant = $montant;

        return $this;
    }


}
