<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\precision\CreatePrecision;
use App\Controller\precision\ListePrecision;
use App\Repository\PrecisionRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: PrecisionRepository::class)]
#[ORM\Table(name: '`precision`')]
#[ApiResource(
    collectionOperations: [
          'liste_precision' => [
            'method' => 'GET',
            'path' => '/precisions',
            'controller' => ListePrecision::class
           ],
           'create_precision' => [
            'method' => 'POST',
            'path' => '/precisions',
            'controller' => CreatePrecision::class
           ]
],
    itemOperations: ['get','put','delete'],
    attributes: ['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    denormalizationContext: ['groups' => ['pr:write', 'app:write', 'bsr:write']],
    normalizationContext: ['groups' => ['pr:read','app:read'],'enable_max_depth' => true, // If using Max Depth
     'skip_null_values' => false ]
)]
class Precision
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['pr:read', 'pr:write', 'app:write', 'bsr:write','app:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['pr:read','pr:write', 'app:write', 'bsr:write','app:read'])]
    private ?string $libelle = null;

    #[ORM\Column(type: Types::DECIMAL, precision: 10, scale: '0', nullable: true)]
    #[Groups(['pr:read','pr:write', 'app:write', 'bsr:write','app:read'])]
    private ?string $tarif = null;

    #[ORM\ManyToMany(targetEntity: Application::class, mappedBy: 'precisions', cascade: ['persist', 'remove'])]
    private Collection $applications;

    public function __construct()
    {
        $this->applications = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getLibelle(): ?string
    {
        return $this->libelle;
    }

    public function setLibelle(?string $libelle): static
    {
        $this->libelle = $libelle;

        return $this;
    }

    public function getTarif(): ?string
    {
        return $this->tarif;
    }

    public function setTarif(?string $tarif): static
    {
        $this->tarif = $tarif;

        return $this;
    }



    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, Application>
     */
    public function getApplications(): Collection
    {
        return $this->applications;
    }

    public function addApplication(Application $application): static
    {
        if (!$this->applications->contains($application)) {
            $this->applications->add($application);
            $application->addPrecision($this);
        }

        return $this;
    }

    public function removeApplication(Application $application): static
    {
        if ($this->applications->removeElement($application)) {
            $application->removePrecision($this);
        }

        return $this;
    }
}
