<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\ramification\CreateRamification;
use App\Controller\ramification\ListRamification;
use App\Controller\ramification\RamificationChilds;
use App\Repository\RamificationRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: RamificationRepository::class)]
#[ApiResource(
    collectionOperations: [

        'create_ramifications' => [
            'method' => 'POST',
            'path' => '/ramifications',
            'controller' => CreateRamification::class,
        ],
        'list_ramifications' => [
            'method' => 'GET',
            'path' => '/ramifications',
            'controller' => ListRamification::class,
        ],
        'ramification_childs' => [
            'method' => 'GET',
            'path' => '/ramifications/{id}/childs',
            'controller' => RamificationChilds::class,
        ],

    ],
    itemOperations: [
        'get',
        'put',
        'delete',
    ],
    attributes: ["route_prefix"=>"/v1", "pagination_client_items_per_page"=>true],
    normalizationContext: ['groups' => ['ram:read', 'ram:details:read','contrib:details:read', 'tram:details:read'], 'skip_null_values' => false]

)]
class Ramification
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups([ 'ram:read', 'ram:details:read'])]

    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups([ 'ram:read', 'ram:details:read'])]

    private ?string $nom = null;

    #[ORM\Column(length: 255)]
    #[Groups([ 'ram:read', 'ram:details:read'])]
    private ?string $code = null;

    #[Groups([ 'ram:read', 'ram:details:read'])]
    private ?array $branches = [];

    #[ORM\Column]
    #[Groups([ 'ram:read', 'ram:details:read'])]

    private ?int $cptr = 1;

    #[ORM\Column]
    #[Groups([ 'ram:read', 'ram:details:read'])]
    private ?int $cptrEquipement = 1;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'ram:read', 'ram:details:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'ram:read', 'ram:details:read'])]

    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\Column]
    #[Groups([ 'ram:read', 'ram:details:read', 'aff:details:read'])]
    private ?bool $enabled = null;



    #[ORM\Column(nullable: true)]
    #[Groups([ 'ram:read', 'ram:details:read'])]
    private ?int $ordre = null;

    #[ORM\ManyToOne(targetEntity: self::class, inversedBy: 'childs')]
    #[Groups([ 'ram:read', 'ram:details:read', 'aff:details:read'])]
    private ?self $parent = null;

    #[ORM\OneToMany(mappedBy: 'parent', targetEntity: self::class)]
    private Collection $childs;




    #[ORM\ManyToOne(inversedBy: 'ramifications')]
    #[Groups([ 'tram:read', 'tram:details:read', 'aff:details:read'])]
    private ?TypeRamification $type = null;

    #[ORM\OneToMany(mappedBy: 'ramification', targetEntity: Usager::class)]
    private Collection $usagers;

    #[ORM\OneToMany(mappedBy: 'ramification', targetEntity: Application::class)]
    private Collection $applications;



    public function __construct()
    {
        $this->childs = new ArrayCollection();
        $this->usagers = new ArrayCollection();
        $this->applications = new ArrayCollection();
    }



    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): static
    {
        $this->code = $code;

        return $this;
    }

    public function getCptr(): ?int
    {
        return $this->cptr;
    }

    public function setCptr(int $cptr): static
    {
        $this->cptr = $cptr;

        return $this;
    }

    public function getCptrEquipement(): ?int
    {
        return $this->cptrEquipement;
    }

    public function setCptrEquipement(int $cptrEquipement): static
    {
        $this->cptrEquipement = $cptrEquipement;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }



    public function getOrdre()
    {
        return $this->ordre;
    }

    public function setOrdre( $ordre)
    {
        $this->ordre = $ordre;

        return $this;
    }

    public function getParent(): ?self
    {
        return $this->parent;
    }

    public function setParent(?self $parent): static
    {
        $this->parent = $parent;

        return $this;
    }

    /**
     * @return Collection<int, self>
     */
    public function getChilds(): Collection
    {
        return $this->childs;
    }

    public function addChild(self $child): static
    {
        if (!$this->childs->contains($child)) {
            $this->childs->add($child);
            $child->setParent($this);
        }

        return $this;
    }

    public function removeChild(self $child): static
    {
        if ($this->childs->removeElement($child)) {
            // set the owning side to null (unless already changed)
            if ($child->getParent() === $this) {
                $child->setParent(null);
            }
        }

        return $this;
    }


    public function getType(): ?TypeRamification
    {
        return $this->type;
    }

    public function setType(?TypeRamification $type): static
    {
        $this->type = $type;

        return $this;
    }

    /**
     * @return Collection<int, Usager>
     */
    public function getUsagers(): Collection
    {
        return $this->usagers;
    }

    public function addUsager(Usager $usager): static
    {
        if (!$this->usagers->contains($usager)) {
            $this->usagers->add($usager);
            $usager->setRamification($this);
        }

        return $this;
    }

    public function removeUsager(Usager $usager): static
    {
        if ($this->usagers->removeElement($usager)) {
            // set the owning side to null (unless already changed)
            if ($usager->getRamification() === $this) {
                $usager->setRamification(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Application>
     */
    public function getApplications(): Collection
    {
        return $this->applications;
    }

    public function addApplication(Application $application): static
    {
        if (!$this->applications->contains($application)) {
            $this->applications->add($application);
            $application->setRamification($this);
        }

        return $this;
    }

    public function removeApplication(Application $application): static
    {
        if ($this->applications->removeElement($application)) {
            // set the owning side to null (unless already changed)
            if ($application->getRamification() === $this) {
                $application->setRamification(null);
            }
        }

        return $this;
    }




}
