<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\transactions\CheckTransaction;
use App\Controller\transactions\CollecteSituation;
use App\Controller\transactions\CreateCollecte;
use App\Controller\transactions\CreateTransaction;
use App\Controller\transactions\DashboardChart;
use App\Controller\transactions\DashboardPieChart;
use App\Controller\transactions\ListeCollecte;
use App\Controller\transactions\ListeTransactions;
use App\Repository\TransactionRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: TransactionRepository::class)]
#[ApiResource(
    collectionOperations: [
        'list_transactions' => [
            'method' => 'GET',
            'path' => '/transactions',
            'controller' => ListeTransactions::class,
        ],
        'create_transactions' => [
            'method' => 'POST',
            'path' => '/transactions',
            'controller' => CreateTransaction::class,
        ],

        'check_transactions' => [
            'method' => 'GET',
            'path' => '/transactions/{id}/trans-check',
            'controller' => CheckTransaction::class,
        ],


    ],
    itemOperations: [
        'get',
        'delete',
        'put'

    ],
    attributes: ["route_prefix"=>"/v1", "pagination_client_items_per_page"=>true],
    normalizationContext: ["skip_null_values" => false,'groups' => ['trans:read', 'trans:details:read', 'usager:details:read', 'usr:details:read', 'mp:read', 'ech:details:read', 'crt:details:read']]
)]
class Transaction
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?int $id = null;

    #[ORM\ManyToOne(inversedBy: 'transactions')]
    #[Groups([ 'trans:read', 'trans:details:read', 'trans:read'])]

    private ?Usager $usager = null;

    #[ORM\Column(type: Types::DECIMAL, precision: 10, scale: '0')]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?string $montant = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?string $statut = null;

    #[ORM\ManyToOne(inversedBy: 'transactions')]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?Echeance $echeance = null;

    #[ORM\ManyToOne(inversedBy: 'transactions')]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?User $createdBy = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?string $orderId = null;

    #[ORM\ManyToOne(inversedBy: 'transactions')]
    #[Groups([ 'trans:read', 'trans:details:read', 'mp:read'])]

    private ?MoyenPaiement $moyenPaiement = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?string $payToken = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?string $paymentUrl = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups([ 'trans:read', 'trans:details:read'])]

    private ?string $notifToken = null;

    #[ORM\ManyToMany(targetEntity: Paiement::class, mappedBy: 'transactions')]
    #[Groups([ 'trans:read', 'trans:details:read'])]
    private Collection $paiements;

    public function __construct()
    {
        $this->paiements = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getUsager(): ?Usager
    {
        return $this->usager;
    }

    public function setUsager(?Usager $usager): static
    {
        $this->usager = $usager;

        return $this;
    }

    public function getMontant(): ?string
    {
        return $this->montant;
    }

    public function setMontant(string $montant): static
    {
        $this->montant = $montant;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getStatut(): ?string
    {
        return $this->statut;
    }

    public function setStatut(string $statut): static
    {
        $this->statut = $statut;

        return $this;
    }

    public function getEcheance(): ?Echeance
    {
        return $this->echeance;
    }

    public function setEcheance(?Echeance $echeance): static
    {
        $this->echeance = $echeance;

        return $this;
    }

    public function getCreatedBy(): ?User
    {
        return $this->createdBy;
    }

    public function setCreatedBy(?User $createdBy): static
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    public function getOrderId(): ?string
    {
        return $this->orderId;
    }

    public function setOrderId(?string $orderId): static
    {
        $this->orderId = $orderId;

        return $this;
    }

    public function getMoyenPaiement(): ?MoyenPaiement
    {
        return $this->moyenPaiement;
    }

    public function setMoyenPaiement(?MoyenPaiement $moyenPaiement): static
    {
        $this->moyenPaiement = $moyenPaiement;

        return $this;
    }

    public function getPayToken(): ?string
    {
        return $this->payToken;
    }

    public function setPayToken(?string $payToken): static
    {
        $this->payToken = $payToken;

        return $this;
    }

    public function getPaymentUrl(): ?string
    {
        return $this->paymentUrl;
    }

    public function setPaymentUrl(?string $paymentUrl): static
    {
        $this->paymentUrl = $paymentUrl;

        return $this;
    }

    public function getNotifToken(): ?string
    {
        return $this->notifToken;
    }

    public function setNotifToken(?string $notifToken): static
    {
        $this->notifToken = $notifToken;

        return $this;
    }

    /**
     * @return Collection<int, Paiement>
     */
    public function getPaiements(): Collection
    {
        return $this->paiements;
    }

    public function addPaiement(Paiement $paiement): static
    {
        if (!$this->paiements->contains($paiement)) {
            $this->paiements->add($paiement);
            $paiement->addTransaction($this);
        }

        return $this;
    }

    public function removePaiement(Paiement $paiement): static
    {
        if ($this->paiements->removeElement($paiement)) {
            $paiement->removeTransaction($this);
        }

        return $this;
    }
    public function setPaiements($paiements){

        $this->paiements = $paiements;
        return $this;

    }
}
