<?php

namespace App\Entity;

use App\Controller\type_ramifications\CreateTypeRamification;
use App\Controller\type_ramifications\ListTypeRamification;
use App\Repository\TypeRamificationRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Annotation\ApiResource;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: TypeRamificationRepository::class)]
#[ApiResource(
    collectionOperations: [
        'liste_type_ramification' => [
            'method' => 'GET',
            'path' => '/type_ramifications',
            'controller' => ListTypeRamification::class,
        ],
        'create_type_ramification' => [
            'method' => 'POST',
            'path' => '/type_ramifications',
            'controller' => CreateTypeRamification::class,
        ]
    ],
    itemOperations: [
        "get",
        "put",
        "delete",

    ],
    attributes: ["route_prefix"=>"/v1", "pagination_client_items_per_page"=>true],
    normalizationContext: ["skip_null_values" => false,'groups' => ['tram:read', 'tram:details:read', 'ent:details:read']]
)]
class TypeRamification
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups([ 'tram:read', 'tram:details:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups([ 'tram:read', 'tram:details:read'])]
    private ?string $nom = null;

    #[ORM\Column]
    #[Groups([ 'tram:read', 'tram:details:read'])]
    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'tram:read', 'tram:details:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups([ 'tram:read', 'tram:details:read'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\OneToMany(mappedBy: 'type', targetEntity: Ramification::class)]
    private Collection $ramifications;

    #[ORM\Column]
    #[Groups([ 'tram:read', 'tram:details:read'])]

    private ?int $ordre = null;



    public function __construct()
    {
        $this->ramifications = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, Ramification>
     */
    public function getRamifications(): Collection
    {
        return $this->ramifications;
    }

    public function addRamification(Ramification $ramification): static
    {
        if (!$this->ramifications->contains($ramification)) {
            $this->ramifications->add($ramification);
            $ramification->setType($this);
        }

        return $this;
    }

    public function removeRamification(Ramification $ramification): static
    {
        if ($this->ramifications->removeElement($ramification)) {
            // set the owning side to null (unless already changed)
            if ($ramification->getType() === $this) {
                $ramification->setType(null);
            }
        }

        return $this;
    }

    public function getOrdre(): ?int
    {
        return $this->ordre;
    }

    public function setOrdre(int $ordre): static
    {
        $this->ordre = $ordre;

        return $this;
    }


}
