<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\typeUser\CreateTypeUser;
use App\Controller\typeUser\ListeTypeUser;
use App\Repository\TypeUserRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: TypeUserRepository::class)]
#[ApiResource(
    collectionOperations: [
         'liste_type_user' => [
            'method' => 'GET',
            'path' => '/type_users',
            'controller' => ListeTypeUser::class
         ],
         'create_type_user' => [
            'method' => 'POST',
            'path' => '/type_users',
            'controller' => CreateTypeUser::class
         ]
         ],
         attributes: ['route_prefix' => '/v1',
         'pagination_client_items_per_page' => true
        ],
        normalizationContext: [ 'groups' => ['typeUser:read'],'enable_max_depth' => true, // If using Max Depth
        'skip_null_values' => false ],// Ensure null values are not skipped],
        denormalizationContext: ['groups' => ['typeUser:write']]
         
)]
class TypeUser
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['typeUser:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups(['typeUser:read','typeUser:write', 'user:read'])]
    private ?string $nom = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['typeUser:read','typeUser:write', 'user:read'])]
    private ?string $description = null;

    #[ORM\Column]
    #[Groups(['typeUser:read','typeUser:write', 'user:read'])]
    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['typeUser:read','typeUser:write', 'user:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['typeUser:read','typeUser:write', 'user:read'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\OneToMany(mappedBy: 'type', targetEntity: User::class)]
    #[Groups(['typeUser:read','typeUser:write'])]
    private Collection $users;

    public function __construct()
    {
        $this->users = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): static
    {
        $this->description = $description;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    /**
     * @return Collection<int, User>
     */
    public function getUsers(): Collection
    {
        return $this->users;
    }

    public function addUser(User $user): static
    {
        if (!$this->users->contains($user)) {
            $this->users->add($user);
            $user->setType($this);
        }

        return $this;
    }

    public function removeUser(User $user): static
    {
        if ($this->users->removeElement($user)) {
            // set the owning side to null (unless already changed)
            if ($user->getType() === $this) {
                $user->setType(null);
            }
        }

        return $this;
    }
}
