<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\usager\CreateUsager;
use App\Controller\usager\ListeUsager;
use App\Controller\usager\RedevableEcheances;
use App\Controller\usager\SearchUsagers;
use App\Repository\UsagerRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: UsagerRepository::class)]
#[ApiResource(
    collectionOperations: [
          'liste_usager' => [
            'method' => 'GET',
            'path' => '/usagers',
            'controller' => ListeUsager::class
           ],
           'create_usager' => [
            'method' => 'POST',
            'path' => '/usagers',
            'controller' => CreateUsager::class
           ],
        'search_contribuable' => [
            'method' => 'GET',
            'path' => '/usagers/{key}/search',
            'controller' => SearchUsagers::class
        ],

           
      
],
    itemOperations: ['get','put','delete'],
    attributes:['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    normalizationContext: ['groups' => ['usager:read'],'enable_max_depth' => true, // If using Max Depth
                                                      'skip_null_values' => false ],

)]
class Usager
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $type = null;

    #[ORM\Column(length: 255,nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $nom = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $prenom = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $adresse = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $raisonSociale = null;

    #[ORM\Column(length: 255)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $telephone = null;

    #[ORM\Column]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?bool $enabled = false;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\ManyToOne(inversedBy: 'usagers')]
    #[Groups(['usager:read','act:read'])]
    private ?Ville $ville = null;

    #[ORM\ManyToOne(inversedBy: 'usagers')]
    #[Groups(['usager:read','act:read'])]
    private ?Activite $activite = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]
    private ?string $numPieceIdentite = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['usager:read','ville:read','act:read'])]

    private ?string $typePieceIdentite = null;

    #[ORM\ManyToOne(inversedBy: 'usagers')]
    private ?User $createdBy = null;

    #[ORM\OneToMany(mappedBy: 'usager', targetEntity: Enrollement::class)]
    private Collection $enrollements;

    #[ORM\ManyToOne(inversedBy: 'usagers')]
    private ?Ramification $ramification = null;

    #[ORM\OneToMany(mappedBy: 'usager', targetEntity: Echeance::class)]
    private Collection $echeances;

    #[Groups(['usager:read','ville:read','act:read'])]
    private $situation;

    #[Groups(['usager:read'])]
    private $fullName;

    #[ORM\OneToMany(mappedBy: 'usager', targetEntity: Transaction::class)]
    private Collection $transactions;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['act:read','usager:read','bsr:read'])]
    private ?string $numBumda = null;

    public function __construct()
    {
        $this->enrollements = new ArrayCollection();
        $this->echeances = new ArrayCollection();
        $this->transactions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(string $type): static
    {
        $this->type = $type;

        return $this;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function getPrenom(): ?string
    {
        return $this->prenom;
    }

    public function setPrenom(string $prenom): static
    {
        $this->prenom = $prenom;

        return $this;
    }

    public function getAdresse(): ?string
    {
        return $this->adresse;
    }

    public function setAdresse(?string $adresse): static
    {
        $this->adresse = $adresse;

        return $this;
    }

    public function getRaisonSociale(): ?string
    {
        return $this->raisonSociale;
    }

    public function setRaisonSociale(?string $raisonSociale): static
    {
        $this->raisonSociale = $raisonSociale;

        return $this;
    }

    public function getTelephone(): ?string
    {
        return $this->telephone;
    }

    public function setTelephone(string $telephone): static
    {
        $this->telephone = $telephone;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getVille(): ?Ville
    {
        return $this->ville;
    }

    public function setVille(?Ville $ville): static
    {
        $this->ville = $ville;

        return $this;
    }

    public function getActivite(): ?Activite
    {
        return $this->activite;
    }

    public function setActivite(?Activite $activite): static
    {
        $this->activite = $activite;

        return $this;
    }

    public function getCreatedBy(): ?User
    {
        return $this->createdBy;
    }

    public function setCreatedBy(?User $createdBy): static
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    /**
     * @return Collection<int, Enrollement>
     */
    public function getEnrollements(): Collection
    {
        return $this->enrollements;
    }

    public function addEnrollement(Enrollement $enrollement): static
    {
        if (!$this->enrollements->contains($enrollement)) {
            $this->enrollements->add($enrollement);
            $enrollement->setUsager($this);
        }

        return $this;
    }

    public function removeEnrollement(Enrollement $enrollement): static
    {
        if ($this->enrollements->removeElement($enrollement)) {
            // set the owning side to null (unless already changed)
            if ($enrollement->getUsager() === $this) {
                $enrollement->setUsager(null);
            }
        }

        return $this;
    }

    public function getRamification(): ?Ramification
    {
        return $this->ramification;
    }

    public function setRamification(?Ramification $ramification): static
    {
        $this->ramification = $ramification;

        return $this;
    }

    /**
     * @return string|null
     */
    public function getNumPieceIdentite(): ?string
    {
        return $this->numPieceIdentite;
    }

    /**
     * @param string|null $numPieceIdentite
     */
    public function setNumPieceIdentite(?string $numPieceIdentite): void
    {
        $this->numPieceIdentite = $numPieceIdentite;
    }

    /**
     * @return string|null
     */
    public function getTypePieceIdentite(): ?string
    {
        return $this->typePieceIdentite;
    }

    /**
     * @param string|null $typePieceIdentite
     */
    public function setTypePieceIdentite(?string $typePieceIdentite): void
    {
        $this->typePieceIdentite = $typePieceIdentite;
    }

    /**
     * @return Collection<int, Echeance>
     */
    public function getEcheances(): Collection
    {
        return $this->echeances;
    }

    public function addEcheance(Echeance $echeance): static
    {
        if (!$this->echeances->contains($echeance)) {
            $this->echeances->add($echeance);
            $echeance->setUsager($this);
        }

        return $this;
    }

    public function removeEcheance(Echeance $echeance): static
    {
        if ($this->echeances->removeElement($echeance)) {
            // set the owning side to null (unless already changed)
            if ($echeance->getUsager() === $this) {
                $echeance->setUsager(null);
            }
        }

        return $this;
    }

    public function getSituation(){
        $prevu = 0;
        $payer = 0;
        $du = 0;
        $restant = 0;

        foreach ($this->getEcheances() as $echeance) {
            if ($echeance->getAnnee() == date('Y')) {
                $prevu += (int)$echeance->getMontant();
            }else{
                $du += (int)$echeance->getMontant();
            }
            /** @var Transaction $transaction */

        }
        foreach ($this->getTransactions() as $transaction){

            if($transaction->getCreatedAt()->format('Y') == date('Y')){
                //dd($transaction->getCreatedAt()->format('Y') );

                if($transaction->getStatut() == "Confirmé" ){
                    $payer += $transaction->getMontant();
                }
            }
        }

        $restant = $prevu-$payer;


        return [
            'prevu' => $prevu,
            'payer' => $payer,
            'restant' => $restant,
            'du' => $du,
        ];


    }

    public function getFullName(){
        if (empty($this->getRaisonSociale()))
            return $this->prenom. " ". $this->nom;
        else
            return $this->raisonSociale;
    }


    /**
     * @return Collection<int, Transaction>
     */
    public function getTransactions(): Collection
    {
        return $this->transactions;
    }

    public function addTransaction(Transaction $transaction): static
    {
        if (!$this->transactions->contains($transaction)) {
            $this->transactions->add($transaction);
            $transaction->setUsager($this);
        }

        return $this;
    }

    public function removeTransaction(Transaction $transaction): static
    {
        if ($this->transactions->removeElement($transaction)) {
            // set the owning side to null (unless already changed)
            if ($transaction->getUsager() === $this) {
                $transaction->setUsager(null);
            }
        }

        return $this;
    }

    /**
     * @return string|null
     */
    public function getNumBumda(): ?string
    {
        return $this->numBumda;
    }

    /**
     * @param string|null $numBumda
     */
    public function setNumBumda(?string $numBumda): void
    {
        $this->numBumda = $numBumda;
    }

}
