<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\user\CheckUser;
use App\Controller\user\CreateUser;
use App\Controller\user\ListeUser;
use App\Repository\UserRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: UserRepository::class)]
#[ORM\Table(name: '`user`')]
#[ApiResource(
    collectionOperations: [
          'liste_user' => [
            'method' => 'GET',
            'path' => '/users',
            'controller' => ListeUser::class
    ],
         'create_user' => [
            'method' => 'POST',
            'path' => '/users',
            'controller' => CreateUser::class
         ],
         'check_users' => [
            'method' => 'POST',
            'path' => '/users/check',
            'controller' => CheckUser::class,
        ]
],
    itemOperations: ['get','put','delete'],
    attributes: ['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    denormalizationContext: ['groups' => ['user:write']],
    normalizationContext: ['groups' => ['user:read'],'enable_max_depth' => true, // If using Max Depth
                                                      'skip_null_values' => false ]

)]
class User implements UserInterface, PasswordAuthenticatedUserInterface
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['user:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 180, unique: true)]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private ?string $email = null;

    #[ORM\Column]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private array $roles = [];

    /**
     * @var string The hashed password
     */
    #[ORM\Column]
    #[Groups(['user:write','typeUser:read'])]
    private ?string $password = null;

    #[ORM\Column(length: 255)]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private ?string $nom = null;

    #[ORM\Column(length: 255)]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private ?string $prenom = null;

    #[Groups(['user:read','typeUser:read'])]
    private ?string $fullName = null;


    #[ORM\Column]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['user:read','user:write','typeUser:read'])]
    private ?\DateTimeInterface $updatedAt = null;
    
    #[ORM\ManyToOne(inversedBy: 'users')]
    #[Groups(['user:read','user:write'])]
    private ?TypeUser $type = null;

    #[ORM\Column(length: 255)]
    #[Groups(['user:read','user:write'])]
    private ?string $username = null;

    #[ORM\OneToMany(mappedBy: 'createdBy', targetEntity: Usager::class)]
    private Collection $usagers;

    #[ORM\Column(type: Types::DATETIME_MUTABLE, nullable: true)]
    private ?\DateTimeInterface $lastLogin = null;

    #[ORM\OneToMany(mappedBy: 'createdBy', targetEntity: Enrollement::class)]
    private Collection $enrollements;

    #[ORM\OneToMany(mappedBy: 'createdBy', targetEntity: Transaction::class)]
    private Collection $transactions;

    public function __construct()
    {
        $this->usagers = new ArrayCollection();
        $this->enrollements = new ArrayCollection();
        $this->transactions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(string $email): static
    {
        $this->email = $email;

        return $this;
    }

    /**
     * A visual identifier that represents this user.
     *
     * @see UserInterface
     */
    public function getUserIdentifier(): string
    {
        return (string) $this->email;
    }

    /**
     * @see UserInterface
     */
    public function getRoles(): array
    {
        $roles = $this->roles;
        // guarantee every user at least has ROLE_USER
        $roles[] = 'ROLE_USER';

        return array_unique($roles);
    }

    public function setRoles(array $roles): static
    {
        $this->roles = $roles;

        return $this;
    }

    /**
     * @see PasswordAuthenticatedUserInterface
     */
    public function getPassword(): string
    {
        return $this->password;
    }

    public function setPassword(string $password): static
    {
        $this->password = $password;

        return $this;
    }

    /**
     * @see UserInterface
     */
    public function eraseCredentials(): void
    {
        // If you store any temporary, sensitive data on the user, clear it here
        // $this->plainPassword = null;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function getPrenom(): ?string
    {
        return $this->prenom;
    }

    public function setPrenom(string $prenom): static
    {
        $this->prenom = $prenom;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getType(): ?TypeUser
    {
        return $this->type;
    }

    public function setType(?TypeUser $type): static
    {
        $this->type = $type;

        return $this;
    }

    public function getUsername(): ?string
    {
        return $this->username;
    }

    public function setUsername(string $username): static
    {
        $this->username = $username;

        return $this;
    }

    /**
     * @return Collection<int, Usager>
     */
    public function getUsagers(): Collection
    {
        return $this->usagers;
    }

    public function addUsager(Usager $usager): static
    {
        if (!$this->usagers->contains($usager)) {
            $this->usagers->add($usager);
            $usager->setCreatedBy($this);
        }

        return $this;
    }

    public function removeUsager(Usager $usager): static
    {
        if ($this->usagers->removeElement($usager)) {
            // set the owning side to null (unless already changed)
            if ($usager->getCreatedBy() === $this) {
                $usager->setCreatedBy(null);
            }
        }

        return $this;
    }

    public function getLastLogin(): ?\DateTimeInterface
    {
        return $this->lastLogin;
    }

    public function setLastLogin(?\DateTimeInterface $lastLogin): static
    {
        $this->lastLogin = $lastLogin;

        return $this;
    }

    /**
     * @return Collection<int, Enrollement>
     */
    public function getEnrollements(): Collection
    {
        return $this->enrollements;
    }

    public function addEnrollement(Enrollement $enrollement): static
    {
        if (!$this->enrollements->contains($enrollement)) {
            $this->enrollements->add($enrollement);
            $enrollement->setCreatedBy($this);
        }

        return $this;
    }

    public function removeEnrollement(Enrollement $enrollement): static
    {
        if ($this->enrollements->removeElement($enrollement)) {
            // set the owning side to null (unless already changed)
            if ($enrollement->getCreatedBy() === $this) {
                $enrollement->setCreatedBy(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Transaction>
     */
    public function getTransactions(): Collection
    {
        return $this->transactions;
    }

    public function addTransaction(Transaction $transaction): static
    {
        if (!$this->transactions->contains($transaction)) {
            $this->transactions->add($transaction);
            $transaction->setCreatedBy($this);
        }

        return $this;
    }

    public function removeTransaction(Transaction $transaction): static
    {
        if ($this->transactions->removeElement($transaction)) {
            // set the owning side to null (unless already changed)
            if ($transaction->getCreatedBy() === $this) {
                $transaction->setCreatedBy(null);
            }
        }

        return $this;
    }

    /**
     * @return string|null
     */
    public function getFullName(): ?string
    {
        return $this->fullName;
    }

    /**
     * @param string|null $fullName
     */
    public function setFullName(?string $fullName): void
    {
        $this->fullName = $fullName;
    }

}
