<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Controller\ville\CreateVille;
use App\Controller\ville\ListeVille;
use App\Repository\VilleRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

#[ORM\Entity(repositoryClass: VilleRepository::class)]
#[ApiResource(
    collectionOperations: [
          'liste_ville' => [
            'method' => 'GET',
            'path' => '/villes',
            'controller' => ListeVille::class
           ],
           'create_ville' => [
            'method' => 'POST',
            'path' => '/villes',
            'controller' => CreateVille::class
           ]
           
      
],
    itemOperations: ['get','put','delete'],
    attributes:['route_prefix' => '/v1','pagination_client_items_per_page' => true],
    normalizationContext: ['groups' => ['ville:read'],'enable_max_depth' => true, // If using Max Depth
                                                      'skip_null_values' => false ],
    denormalizationContext: ['groups' => ['ville:write']]

)]
class Ville
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    #[Groups(['ville:read','pays:read','ville:read','usager:read','pays:read','app:read'])]
    private ?int $id = null;

    #[ORM\Column(length: 255)]
    #[Groups(['ville:read','ville:write','pays:read','ville:read','usager:read','pays:read','app:read'])]
    private ?string $nom = null;

    #[ORM\Column(length: 255, nullable: true)]
    #[Groups(['ville:read','ville:write','pays:read','ville:read','usager:read','pays:read','app:read'])]
    private ?string $description = null;

    #[ORM\Column]
    #[Groups(['ville:read','ville:write','pays:read','ville:read','usager:read','pays:read','app:read'])]
    private ?bool $enabled = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['ville:read','ville:write','pays:read','ville:read','usager:read','pays:read','app:read'])]
    private ?\DateTimeInterface $createdAt = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    #[Groups(['ville:read','ville:write','pays:read','ville:read','usager:read','pays:read','app:read'])]
    private ?\DateTimeInterface $updatedAt = null;

    #[ORM\ManyToOne(inversedBy: 'villes')]
    #[Groups(['ville:read','ville:write','usager:read','pays:read','app:read'])]
    private ?Pays $pays = null;

    #[ORM\OneToMany(mappedBy: 'ville', targetEntity: Usager::class)]
    #[Groups(['ville:read','ville:write'])]
    private Collection $usagers;

    #[ORM\OneToMany(mappedBy: 'ville', targetEntity: Application::class)]
    #[Groups(['ville:read','ville:write'])]
    private Collection $applications;

    public function __construct()
    {
        $this->usagers = new ArrayCollection();
        $this->applications = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): static
    {
        $this->nom = $nom;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): static
    {
        $this->description = $description;

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): static
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): static
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): static
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getPays(): ?Pays
    {
        return $this->pays;
    }

    public function setPays(?Pays $pays): static
    {
        $this->pays = $pays;

        return $this;
    }

    /**
     * @return Collection<int, Usager>
     */
    public function getUsagers(): Collection
    {
        return $this->usagers;
    }

    public function addUsager(Usager $usager): static
    {
        if (!$this->usagers->contains($usager)) {
            $this->usagers->add($usager);
            $usager->setVille($this);
        }

        return $this;
    }

    public function removeUsager(Usager $usager): static
    {
        if ($this->usagers->removeElement($usager)) {
            // set the owning side to null (unless already changed)
            if ($usager->getVille() === $this) {
                $usager->setVille(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Application>
     */
    public function getApplications(): Collection
    {
        return $this->applications;
    }

    public function addApplication(Application $application): static
    {
        if (!$this->applications->contains($application)) {
            $this->applications->add($application);
            $application->setVille($this);
        }

        return $this;
    }

    public function removeApplication(Application $application): static
    {
        if ($this->applications->removeElement($application)) {
            // set the owning side to null (unless already changed)
            if ($application->getVille() === $this) {
                $application->setVille(null);
            }
        }

        return $this;
    }
}
