<?php

namespace App\Repository;

use App\Entity\Ramification;
use Artprima\QueryFilterBundle\QueryFilter\QueryFilterArgs;
use Artprima\QueryFilterBundle\QueryFilter\QueryResult;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Tools\Pagination\Paginator;


class RamificationRepository extends BaseRepository
{
    public function findByOrderBy(QueryFilterArgs $args): QueryResult
    {
        // Build our request
        $qb = $this->createQueryBuilder('ram')
            ->leftJoin('ram.type', 'tram')
            ->setFirstResult($args->getOffset())
            ->setMaxResults($args->getLimit());

        $proxyQb = $this->pqbManager->wrapQueryBuilder($qb);
        $qb = $proxyQb->getSortedAndFilteredQueryBuilder($args->getSearchBy(), $args->getSortBy());
        $query = $qb->getQuery();
        $paginator = new Paginator($query);

        // return the wrapped result
        return new QueryResult($paginator->getIterator()->getArrayCopy(), count($paginator));
    }
    public function findAllBranchesForRamification(Ramification $ramification): ArrayCollection
    {
        $branches = new ArrayCollection();

        // Ajouter la ramification actuelle
        $branches->add($ramification) ;

        // Récupérer les ramifications enfants
        $children = $ramification->getChilds();
        foreach ($children as $child) {
            // Récursion pour récupérer les ramifications liées à cette ramification enfant
            foreach ($this->findAllBranchesForRamification($child) as $sub)
            {
                $branches->add($sub);
            }
        }

        return $branches;
    }
    protected function getEntityClass(): string
    {
        return Ramification::class;
    }
}
