<?php

namespace App\Repository;

use App\Entity\Usager;
use Artprima\QueryFilterBundle\Query\ConditionManager;
use Artprima\QueryFilterBundle\QueryFilter\QueryFilterArgs;
use Artprima\QueryFilterBundle\QueryFilter\QueryResult;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\ORM\Tools\Pagination\Paginator;
use Doctrine\Persistence\ManagerRegistry;

/**
 * @extends ServiceEntityRepository<Usager>
 *
 * @method Usager|null find($id, $lockMode = null, $lockVersion = null)
 * @method Usager|null findOneBy(array $criteria, array $orderBy = null)
 * @method Usager[]    findAll()
 * @method Usager[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class UsagerRepository extends ServiceEntityRepository
{
    public function searchUsager($key){
        return $this->createQueryBuilder('c')

            ->leftJoin('c.ramification', 'r')
            ->where('c.nom like :key or c.prenom like :key or c.raisonSociale like :key or c.telephone like :key or c.numBumda like :key')
            ->setParameters([
                'key' => '%'.$key.'%',
            ])
            ->getQuery()
            ->getResult();
    }

    /**
     * @var ConditionManager
     */
    private $pqbManager;
    public function __construct(ManagerRegistry $registry, ConditionManager $manager)
    {
        parent::__construct($registry, Usager::class);
        $this->pqbManager = $manager;
    }
    public function findByOrderBy(QueryFilterArgs $args): QueryResult
    {
        // Build our request
        $qb = $this->createQueryBuilder('usager')
            ->setFirstResult($args->getOffset())
            ->setMaxResults($args->getLimit());

        $proxyQb = $this->pqbManager->wrapQueryBuilder($qb);
        $qb = $proxyQb->getSortedAndFilteredQueryBuilder($args->getSearchBy(), $args->getSortBy());
        $query = $qb->getQuery();
        $paginator = new Paginator($query);

        // return the wrapped result
        return new QueryResult($paginator->getIterator()->getArrayCopy(), count($paginator));
    }    

//    /**
//     * @return Usager[] Returns an array of Usager objects
//     */
//    public function findByExampleField($value): array
//    {
//        return $this->createQueryBuilder('u')
//            ->andWhere('u.exampleField = :val')
//            ->setParameter('val', $value)
//            ->orderBy('u.id', 'ASC')
//            ->setMaxResults(10)
//            ->getQuery()
//            ->getResult()
//        ;
//    }

//    public function findOneBySomeField($value): ?Usager
//    {
//        return $this->createQueryBuilder('u')
//            ->andWhere('u.exampleField = :val')
//            ->setParameter('val', $value)
//            ->getQuery()
//            ->getOneOrNullResult()
//        ;
//    }
}
